#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import unicodedata

def removeControlChar(string):
	"""
	@brief 제어문자를 제거합니다.
	
	@param string 문자열
	
	@return 경로 문자가 제거된 문자열
	"""
	buffer = ''.join(ch for ch in string if unicodedata.category(ch)[0]!="C")
	return buffer



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals

"""배경 색"""
red = u"\033[41m";           #: 빨강색
green = u"\033[42m";			 #: 초록색
yellow = u"\033[43m";        #: 노랑색
blue = u"\033[44m";          #: 파랑색
magenta = u"\033[45m";       #: 자홍색
cyan = u"\033[46m";          #: 시안색
darkgray = u"\033[100m";     #: 밝은 회색
lightgreen = u"\033[102m";   #: 밝은 초록색



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess

class Notepad(object):
	def open(self):
		"""
		@brief 메모창을 엽니다.
		"""
		self._process = subprocess.Popen(['kate', '--stdin'], stdin=subprocess.PIPE)
		pass
	
	def write(self, content):
		"""
		@brief 메모에 내용을 씁니다.
		@param content 쓸 내용
		"""
		self._process.stdin.write(content)
		pass
	
	def close(self):
		"""
		@brief 메모창을 닫습니다.
		"""
		self._process.kill()
		pass
	
	@classmethod
	def show(cls, content):
		"""
		@brief 주어진 내용을 보여줍니다.
		@param content 보여줄 내용
		@return 객체를 반환합니다.
		"""
		notepad = cls()
		notepad.open()
		notepad.write(content)
		return notepad
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import random
import string as _p_string

def string(length, chars=_p_string.ascii_letters + _p_string.digits):
	"""
	@brief 지정된 길이 만큼의 램덤 문자열을 되돌려줍니다.
	
	@param length 문자열의 길이
	@param chars 문자 집합(리스트 형식)
		[a, ..., z, A, ..., Z, ..., 0, ..., 9] (기본값)
	
	@return 램덤 문자열
	"""
	string = unicode()
	for i in xrange(length):
		string += random.choice(chars)
		pass
	return string
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess

def toBmp(pdf, format='BMP32B', dpi=200):
	"""
	@brief pdf 파일을 bmp로 바꾸어 저장합니다.
	
	@note 참조 :\n
		고스트 스크립트 문서 : http://www.ghostscript.com/doc/current/Readme.htm
	
	@param pdf pdf 바이너리 데이터
		오직 낱장(1 페이지)만 존재하는 pdf 데이터여야 합니다.
	@param format 출력 포멧\n
		BMP16M: 24-bit RGB Color\n
		BMPMONO: Black-and-White Color\n
		BMPGRAY: Grayscale Color\n
		BMPSEP1:\n
		BMPSEP8:\n
		BMP16: 4-bit Color\n
		BMP256: 8-bit Color\n
		BMP32B: 32-bit RGBA Color (기본값)
	@param dpi DPI\n
		기본값은 200DPI입니다.\n
		양의 정수만을 취합니다.
	
	@return BMP 이미지 바이너리 데이터
	"""
	cmd = [
		'gs',
		'-q',
		'-dQUIET',
		'-dSAFER',
		'-dBATCH',
		'-dNOPAUSE',
		'-dNOPROMPT',
		'-dMaxBitmap=500000000',
		'-dAlignToPixels=0',
		'-dGridFitTT=2',
		'-dUseCropBox', # PDF 페이지 박스 내의 내용만 저장
		'-sDEVICE={0}'.format(format), #출력 형식
		'-dTextAlphaBits=4',
		'-dGraphicsAlphaBits=4',
		'-r{0}x{0}'.format(dpi), #DPI
		'-sOutputFile=%stdout', #출력을 표준출력으로
		'-', #입력을 표준입력에서
		]
	process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
	bmp = process.communicate(inData)[0]
	
	return bmp
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
from trashcli.trash import TrashPutCmd
import io

__all__ = ['trashPut']

def trashPut(target):
	"""
	@brief 대상을 휴지통에 버립니다.
	@details trashcli 패키지의 trash-put 명령어에 대한 인터페이스.
	
	@param taget 지울 대상의 경로 문자열
	
	@return True 성공시
	@return False 문제가 존재 할 시
	"""
	out = io.BytesIO()
	err = io.BytesIO()
	type(target)
	TrashPutCmd(out, err).run(['', '--', target.encode('utf8')])
	if err.getvalue():
		return False
	return True



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

def unpadPkcs5(s):
	"""
	@brief PKCS#5 표준 패딩을 제거한다.
	"""
	return s[0:-ord(s[-1])]
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import __builtin__ as builtins
import ANSIColors

def print(*objs) :
	"""
	@brief 디버깅 메시지를 터미널에 출력합니다.
	@details 빨간색으로 강조된 '[디버그]' 문자열 뒤에 주어진 objs가 출력되고 줄바꿈됩니다.
	@link http://pythonhosted.org/ANSIColors-balises/ANSIColors.html ANSIColors Module — ANSIColors-balises public documentation
	"""
	buffer = ANSIColors.sprint('<Red>[디버그]<reset> ')
	builtins.print(buffer, end='');
	builtins.print(*objs);
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
from PyQt4.QtCore import pyqtSlot, QObject
import subprocess

class PulseProgress(QObject):
	def __init__(self, title, message):
		super(PulseProgress, self).__init__()
		self.title = title
		self.message = message
		pass
	
	@pyqtSlot()
	def open(self):
		self._zenity = subprocess.Popen(['zenity', '--progress', '--text', self.message, '--pulsate', '--no-cancel', '--auto-close', '--title', self.title], stdin=subprocess.PIPE)
		self._zenity.stdin.write('y')
		pass
	
	@pyqtSlot()
	def close(self):
		self._zenity.kill()
		pass
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals

__all__ = list()

__all__ += ['reset'] 
reset = u"\033[39;49;00m";       #: 색상값을 초기화

"""글자 색"""
__all__ += ['red', 'green', 'yellow', 'blue', 'blue', 'magenta', 'cyan']
red = u"\033[31;01m";            #: 빨강색
green = u"\033[32;01m";          #: 초록색
yellow = u"\033[33;01m";         #: 노랑색
blue = u"\033[34;01m";           #: 파랑색
magenta = u"\033[35;01m";        #: 자홍색, 마젠타색
cyan = u"\033[36;01m";           #: 밝은 파랑색, 시안색



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import sys
import ANSIColors

__all__ = ['feed', 'pause']

def feed(*msg) :
	"""
	@brief 
	@param msg, ... 화면에 출력한 문자열 또는 출력 옵션 설정
	"""
	ANSIColors.clearLine()
	print(*msg, end='')
	sys.stdout.flush()
	pass

def pause() :
	"""
	@brief 사용자의 입력을 대기합니다.
	"""
	print('[Enter]를 눌러 다음으로 진행합니다...', end = '');
	raw_input("");
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals
import cookielib
import tempfile
import urllib
import re
import chardet
import os

from . import path as _p_path
from . import string as _p_string

def trimFilename(filename, **options):
	"""
	@brief 웹에서 다운받은 파일의 이름을 손질함.
	
	@details \n
		url coding 풀기\n
		사용불가능한 문자를 대체문자로 치환\n
		웹에서의 공백치환을 감지하고 경우에 따라 해제\n
		파일 이름을 다듬기\n
		UHC로 변환가능한 인코딩은 변환
	
	@warning 인코딩 변환이 잘못될수도 있습니다.
	
	@param filename 파일명 유니코드 문자열
	@param from_encoding 입력 인코딩\n
		'auto' : 자동으로 인코딩을 파악합니다. \n
		False : 인코딩을 변환하지 않습니다. (기본)\n
		기타('utf8', 'uhc', ...)
	@param consider_extension 확장자 고려 여부\n
		True: 확장자를 고려하여 작업합니다.\n
		False: 확장자를 고려하지 않습니다. (기본)
		
	@return 변환된 문자열
	@return u'Unknown' : 결과물이 공백이 될 경우 반환됩니다.
	"""
	#옵션 초기값 설정
	options.setdefault(u'consider_extension', False)
	options.setdefault(u'from_encoding', False)
		
	#옵션 처리 : consider_extension
	considerExtension = options[u'consider_extension']
	
	#옵션 처리 : from_encoding
	if options[u'from_encoding'] == u'auto':
		fromEncoding = chardet.detect(filename)['encoding']
	elif options[u'from_encoding'] == False:
		fromEncoding = None
	else:
		fromEncoding = options[u'from_encoding']
	
	#url 디코딩
	filename = urllib.unquote(filename)
	
	#인코딩 변환
	if fromEncoding:
		filename = filename.decode('utf8', errors='replace')
	
	#url 디코딩
	filename = urllib.unquote(filename)
	
	#전체 파일명의 앞뒤 공백 제거
	filename = filename.strip()
	
	#파일명에 사용불가능한 문자 치환
	filename = _p_string.replaceSpiecalChar(filename)
	
	#확장자 분리
	if considerExtension:
		ext = _p_string.extension(filename)
		if ext:
			filename = _p_string.filename(filename)
		else:
			considerExtension = False
		
	#이름에 포함된 and표시 제거
	if not re.search(u' ', filename):
		pattern1 = re.compile(u'[_+]', re.UNICODE)
		pattern2 = re.compile(ur'\.', re.UNICODE)
		if pattern1.search(filename):
			filename = pattern1.sub(' ', filename)
		elif pattern2.search(filename):
			filename = pattern2.sub(' ', filename)
	
	#파일이름의 앞뒤 공백 제거
	filename = filename.strip()
	
	#제어문자 제거
	filename = _p_string.removeControlChar(filename)
	#filename = re.sub('[:cntrl:]', ' ', filename, flags=re.UNICODE)
	
	#파일명 합침.
	if considerExtension:
		filename = '.'.join((filename, ext))
	
	#파일명을 반환
	if filename:
		return filename
	return 'Unknown'
	
def dequoteJsStr(jsStr) :
	"""
	@brief 자바 스크립트를 위해 콰우팅된 문자열을 콰우팅 해제시킵니다.
	@details ex) abc\'asd\' -> abc'asd'
	@return 디콰우팅 된 문자열
	"""
	REGEXS = [
		(r'\\', '\\'),
		(r"\'", "'"),
		(r'\"', '"'),
		(r'\n', '\n')
	];
	for before, after in REGEXS :
		jsStr = jsStr.replace(before, after)
		pass
	return jsStr;

def loadNetscapeCookie(session, cookiePath):
	"""
	@brief Netscape 타입의 쿠키를 가져와서 requests session에 설정합니다.
	
	@remark 쿠키 헤더\n
		# Netscape HTTP Cookie File\n
		# http://www.netscape.com/newsref/std/cookie_spec.html\n
		# This is a generated file!  Do not edit.
	
	@param session requests 세션 객체
	@param cookiePath 쿠키 파일 경로 문자열
	"""
	#임시 파일 생성
	tmpCookiePath = tempfile.mkstemp(prefix='.tmp_', suffix='.cookie')[1]
	
	#임시 파일 객체 생성
	tmpCookie = open(tmpCookiePath, mode='w+b')
	
	#넷스케이프 헤더 작성
	tmpCookie.write('# Netscape HTTP Cookie File\n')
	tmpCookie.write('# http://www.netscape.com/newsref/std/cookie_spec.html\n')
	tmpCookie.write('# This is a generated file!  Do not edit.\n')
	tmpCookie.write('\n')
	
	#기존 쿠키 파일의 내용을 삽입; 윈도우식 줄바꿈 -> 리눅스식으로 치환.
	with open(cookiePath, 'r') as f:
		buffer = f.read().replace('\r\n', '\n')
		tmpCookie.write(buffer)
		pass
	
	#쿠키 묶음 객체를 생성
	cookieJar = cookielib.MozillaCookieJar(tmpCookiePath)
	
	#임시 파일 객체 닫기
	tmpCookie.close()
	
	#쿠키 불러오기
	cookieJar.load(ignore_discard=True, ignore_expires=True)
	
	#세션에 쿠키 설정하기.
	session.cookies = cookieJar
	
	#임시 파일 삭제
	os.remove(tmpCookiePath)
	pass
	


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
@package ufp
@brief 사용자 함수 묶음.
@version v4\n
	bash -> c++ -> (qt ->) python
@author 별님 <w7dn1ng75r@gmail.com>
@details \n
	홈페이지 : http://thestars3.tistory.com/
"""

from __future__ import unicode_literals, absolute_import

__title__ = 'ufp'
__version__ = '1.0.5b2'
__author__ = '별님'
__license__ = 'GPL v3'
__copyright__ = 'Copyright 2015 별님'

from .ufp import *

#하위 호환성 유지를 위한 설정
from . import _compatibleness



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import re
import os.path

try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup

def read(fname):
	return open(os.path.join(os.path.dirname(__file__), fname)).read()

version = ''
with open('ufp/__init__.py', 'r') as fd:
	reg = re.compile(r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]')
	for line in fd:
		m = reg.match(line)
		if m:
			version = m.group(1)
			break
		pass
	pass

if not version:
	raise RuntimeError('Cannot find version information')

buffer = read(os.path.join(os.path.dirname(__file__), 'requirements.txt'))
install_requires = filter(lambda x: x != '', buffer.split('\n'))

setup(
	name             = 'ufp',
	version          = version,
	author           = '별님',
	author_email     = 'w7dn1ng75r@gmail.com',
	url              = 'http://thestars3.tistory.com/',
	description      = 'ufp 라이브러리 python 버전. 각종, 편리한 함수들의 모음.',
	packages         = [
			'ufp',
			'ufp._compatibleness',
			'ufp._compatibleness.terminal',
			'ufp._compatibleness.terminal.color',
			'ufp.gui',
			'ufp.terminal',
		],
    package_dir      = {'ufp': 'ufp'},
    package_data     = {
		'': [
			'README.md',
			'AUTHORS',
			'COPYING'
			]
		},
	zip_safe         = False,
	install_requires = install_requires,
	license          = "GPL v3.0",
	keywords         = ["path", "web", "html", "string", "image", "gui", "termianl"],
	long_description = read('README.md'),
	classifiers      = [
		"Programming Language :: Python :: 2 :: Only",
		"Programming Language :: Python :: 2.7",
		"Intended Audience :: Developers",
		"Development Status :: 5 - Production/Stable",
		"Environment :: X11 Applications",
		"Environment :: Console",
		"Topic :: Terminals :: Terminal Emulators/X Terminals",
		"Topic :: System :: Shells",
		"Topic :: Utilities",
		"Topic :: Text Processing :: General",
		"Topic :: Software Development :: Libraries",
		"Operating System :: Unix",
		"Operating System :: POSIX",
		"Operating System :: MacOS",
		"License :: OSI Approved :: GNU General Public License (GPL)",
		],
	download_url     = "https://github.com/Thestars3/pyufp/releases",
	platforms        = ['Unix', 'POSIX', 'MacOS'],
	include_package_data = True, # True : MANIFEST.in is used
	)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess
import tidylib
import tempfile
import os
import pattern.web

def clean(html, inputEncoding = "utf8") :
	"""
	@brief html 문서를 보다 규격화된 xhtml로 변환합니다.
	@link http://tidy.sourceforge.net/docs/quickref.html HTML Tidy 설정 옵션 빠른 참조
	@link http://countergram.com/open-source/pytidylib/docs/index.html#small-example-of-use Html Tidy에 관한 파이썬 인터페이스 pytidylib.moudule
	"""
	tidyOptions = {
		"output-xhtml": True, #"output-xml" : True
		"quiet": True,
		"show-errors": 0,
		"force-output": True,
		"numeric-entities": True,
		"show-warnings": False,
		"input-encoding": inputEncoding,
		"output-encoding": "utf8",
		"indent": False,
		"tidy-mark": False,
		"wrap": 0
		};
	document, errors = tidylib.tidy_document(html, options = tidyOptions)
	return document

def toText(html, converter='pattern.web') :
	"""
	@brief html 문서를 텍스트 문서로 변환합니다.
	
	@param html 원본 html 텍스트
	@param converter 변환에 사용할 변환기\n
		w3m : w3m 외부 프로그램을 불러와 작업을 하기 때문에 속도가 상당히 느립니다.\n
		pattern.web : pattern(http://www.clips.ua.ac.be/pattern) 라이브러리를 사용합니다. (10개 초과의 공백라인은 자동으로 10개의 공백라인으로 치환됩니다. 앞 뒤 공백은 자동으로 제거됩니다.)
	
	@throw ValueError 지원하지 않는 변환기를 입력 한 경우
	
	@return text
	"""
	if converter == 'w3m':
		tempPath = tempfile.mkstemp(prefix='.tmp_', suffix='.html')[1]
		with open(tempPath, 'w+b') as temp:
			temp.write(html)
			pass
		cmd = ['w3m', '-cols', '98304', '-dump' ,tempPath]
		text = subprocess.check_output(cmd)
		os.remove(tempPath)
		return text
	
	if converter == 'pattern.web':
		text = pattern.web.plaintext(html, linebreaks=10, indentation=True)
		return text
	
	raise ValueError("'{0}'는 지원하지 않는 변환기입니다.".format(converter))



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

def quote(string):
	"""
	@brief shell에서 해석되지 않도록 콰우팅합니다.
	@param string 문자열
	@return 콰우팅 된 문자열
	"""
	buffer = string.replace("'", r"'\''")
	buffer = "'{0}'".format(buffer)
	return buffer
	


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import os
import re
import urllib
import urlparse
import os.path
import datetime
import shutil

def walk(top, maxDepth=None, onerror=None, followlinks=False):
	"""
	@brief 재귀적으로 경로 탐색하기
	@details '.'를 인자로 주었다면, './a', './b' 순으로 점점 더 깊게 탐색합니다.
	@param top 탐색 대상 경로
	@param onerror 오류가 발생하면, 주어진 주소의 함수를 호출하여 오류 인자를 넘겨줍니다.
	@param followlinks 심볼릭 링크된 경로를 포함하여 탐색합니다.
	@param maxDepth 탐색 할 최대 깊이\n
		0이상의 정수여야 함.\n
		0 : 지정된 폴더만
	"""
	top = top.rstrip(os.path.sep)
	assert os.path.isdir(top)
	num_sep = top.count(os.path.sep)
	for root, dirs, files in os.walk(top, onerror=onerror, followlinks=followlinks):
		yield root, dirs, files
		if maxDepth is not None and num_sep + maxDepth <= root.count(os.path.sep):
			del dirs[:]
		pass
	pass
	
def moveAllContent(dirPath, destPath):
	"""
	@brief dirPath에 존재하는 모든 파일을 destPath로 옮깁니다.
	"""
	for i in listdir(dirPath, fullpath=True):
		shutil.move(i, destPath)
		pass
	pass

def mtime(path, format):
	"""
	@brief path에 해당하는 파일의 최근 수정 시각을 지정된 format형식으로 작성하여 반환합니다.
	
	@param path 경로 문자열
	@param format 형식 문자열(datetime strftime에서 지원하는 형식)
	
	@return path에 존재하는 파일의 최근 수정 시각의 format에 지정된 형식
	"""
	buffer = os.path.getmtime(path)
	buffer = datetime.fromtimestamp(buffer).strftime(format)
	return buffer
	
def pjoin(parentPath, filenames):
	"""
	@brief parentPath를 각 filenames의 아이템과 결합합니다.
	
	@param parentPath 부모 경로
	@param filenames 파일명 리스트\n
		[filename, ...]
	
	@return 결합된 리스트\n
		[parentPath/filename, parentPath/filename, ...]
	"""
	result = list()
	for filename in filenames:
		buffer = os.path.join(parentPath, filename)
		result.append(buffer)
		pass
	return result

def listdir(path, **options):
	"""
	@brief 지정된 폴더에서 pattern에 해당하는 파일 및 폴더의 목록을 가져옵니다.
	
	@param path 목록을 가져올 폴더 경로
	@param pattern 필터 패턴(대소문자를 구분하지 않습니다)\n
		string : 패턴 문자열\n
		[string, ...]: 패턴 문자열 리스트\n
		None: 필터링 하지 않음
	@param patternReverse \n
		True : 필터에 매칭되지 않는 대상만 얻습니다.\n
		False : 필터에 매칭되는 대상만 얻습니다.
	@param fullpath 반환시 주어진 경로를 포함한 경로를 반환합니다.\n
		True : [path + filename, ...]\n
		False : [filename, ...]
	@param sortKey 정렬키 함수\n
		해당 함수에는 전체 경로가 전달 됩니다.
	
	@return 필터링된 폴더 내용
	"""
	options.setdefault('pattern', None)
	options.setdefault('patternReverse', False)
	options.setdefault('fullpath', False)
	options.setdefault('sortKey', None)
	
	if options['pattern']:
		pattern = options['pattern']
		pass
		
	#만약 패턴이 문자열이라면 리스트에 담기.
	if isinstance(pattern, unicode):
		pattern = list(pattern)
		pass
	
	#패턴 컴파일
	pattern = map(lambda x: re.compile(x, re.IGNORECASE), pattern)
	
	#목록 작성
	listdir = list()
	for filename in os.listdir(path):
		#패턴 옵션이 설정된 경우 필터링
		if options['pattern']:
			for p in pattern:
				#패턴 역전 옵션이 꺼진 경우
				if p.search(filename) and not options['patternReverse']:
					listdir.append(filename)
					break
				#패턴 역전 옵션이 켜진 경우
				elif options['patternReverse']:
					listdir.append(filename)
					break
				pass
			pass
		pass
	
	#sortKey 옵션이 사용된 경우
	if not options['sortKey']:
		buffer = options['sortKey']
		buffer = lambda x: buffer(os.path.join(path, x))
		listdir.sort(key=buffer)
		pass
	
	#fullpath 옵션이 켜진 경우
	if options['fullpath']:
		listdir = pjoin(path, listdir)
		pass
	
	#결과 반환
	return listdir
	
def toUrl(path):
	"""
	@brief path를 file:///형식으로된 주소로 되돌려줍니다.
	@return file:///형식으로된 주소
	"""
	buffer = path.encode('utf8')
	buffer = urllib.pathname2url(buffer)
	return urlparse.urljoin('file:', buffer)

def replaceSpiecalChar(string, **options) :
	"""
	@brief 운영체제의 경로에서 사용 불가능한 문자열을 대체문자로 치환합니다.
	@details \n
		윈도우 및 유닉스 계열 운영체제에서 파일에 포함되면 문제가 되는 특수문자를 대체문자로 치환합니다.\n
		경로 구분자를 대체 문자로 치환합니다.
	
	@remark \n
		윈도우에서 파일명으로 사용 할  없는 문자들(파일 이름에 다음 문자를 사용할 수 없습니다)\n
		\ / : * ? " < > |
	
	@param type \n
		'windows', 'unix'.\n
		윈도우의 경로 구분 문자는 리눅스에서 표시되는 모양이 다릅니다.\n
		어떤 OS에서 보여지는 모양으로 치환할지 지정합니다.\n
		기본값은 unix입니다.
	@param keep_path_characters\n
		True, False.\n
		경로 구분자를 치환하지 않습니다.\n
		이 설정은 type 설정에 의존합니다.\n
		기본 값은 False입니다.
	"""
	UNIX_PATH_CHARACTER_RE = (u"/", u"／");
	ESCAPE_CHARTER_UNIX_TYPE_RE = (u"\\", u"＼")
	WINDOWS_PATH_CHARACTER_RE = (u"\\", u"￦")
	DEFAULT_REGEXS = [
		(u"?", u"？"),
		(u":", u"："),
		(u"*", u"＊"),
		(u'"', u"＂"),
		(u"<", u"〈"),
		(u">", u"〉"),
		(u"|", u"│"),
		(u"'", u"＇"),
		(u"$", u"＄"),
		(u"!", u"！")
	];
	
	#옵션 초기값 설정
	options.setdefault(u'type', u'unix')
	options.setdefault(u'keep_path_characters', False)
	
	#옵션 처리
	regexs = DEFAULT_REGEXS
	if options[u'type'] == u'unix' :
		regexs.append(ESCAPE_CHARTER_UNIX_TYPE_RE);
		if not options[u'keep_path_characters'] :
			regexs += [UNIX_PATH_CHARACTER_RE];
	elif options[u'type'] == u'windows' :
		regexs += [UNIX_PATH_CHARACTER_RE];
		if not options[u'keep_path_characters'] :
			regexs += [WINDOWS_PATH_CHARACTER_RE];
	
	for before, after in regexs :
		string = string.replace(before, after);
	
	return string;

def dirname(path) :
	"""
	@brief 주어진 경로의 부모 경로를 추출해냅니다.
	@details 만약 'abc'가 주어졌다면, 반환값은 '.'입니다.
	@note os.path.dirname('asd') -> ''가 되는 문제를 해결하기 위해 만들어졌습니다.
	@param path 경로 문자열\n
		주어지는 값은 유니코드 문자열이여야 함.
	"""
	dirnameRe = re.compile(u'(?P<dirname>^.*)/', re.DOTALL | re.UNICODE).search(path);
	if dirnameRe :
		return dirnameRe.group('dirname');
	return '.';

def unique(targetPath, spliteExt = True) :
	"""
	@brief 해당 경로에서 유일한 경로를 만들어 냅니다.
	@details \n
		대소문자는 구분하지 않습니다.\n
		만약, 주어진 경로가 충돌하지 않는다면 주어진 경로를 그대로 반환합니다.\n
		만약 주어진 경로의 부모 경로가 존재하지 않는다면, 주어진 문자열 그대로 반환합니다.\n
		ex) a/b/c -> a/b/c d(1)식으로 충복을 회피처리함.
	@param targetPath 주어지는 값은 유니코드 문자열이여야 함.
	@version v4\n
		BASH -> C++ -> QT -> PYTHON
	@return 유일한 경로
	"""
	#경로 분할
	targetDirname = dirname(targetPath);
	targetBasename = os.path.basename(targetPath);
	
	#부모 경로가 존재하는지 확인
	if not os.path.exists(targetDirname) :
		return targetPath;
	
	#해당 경로의 목록을 작성
	fileList = [u'.', u'..'];
	for dirpath, dirnames, filenames in os.walk(targetDirname) :
		fileList.extend(filenames);
		break;
		
	#중복되는 대상이 존재하는지 확인
	existDuplicateFile = False;
	buffer = re.escape(targetBasename);
	fullmatchRe = re.compile(u"^%(buffer)s$" % locals(), re.DOTALL | re.IGNORECASE | re.UNICODE);
	for fileName in fileList :
		if fullmatchRe.search(fileName) :
			existDuplicateFile = True;
			break;
	
	#중복되는 대상이 존재하고 있지 않다면 이름을 그대로 돌려줌.
	if not existDuplicateFile :
		return targetPath;
		
	#파일명과 확장자를 추출
	if spliteExt :
		targetFileExt = extension(targetBasename);
		if targetFileExt :
			targetFileName = filename(targetBasename);
		else :
			targetFileName = targetBasename;
			spliteExt = False;
	else :
		targetFileName = targetBasename;
	
	#중복 파일들의 숫자를 가져옴.
	escapedTargetFileName = re.escape(targetFileName);
	if spliteExt :
		extractDupCountRe = re.compile(ur"^%(escapedTargetFileName)s \(d(?P<number>[0-9]+)\)\.%(targetFileExt)s$" % locals(), re.DOTALL | re.IGNORECASE | re.UNICODE);
	else :
		extractDupCountRe = re.compile(ur"^%(escapedTargetFileName)s \(d(?P<number>[0-9]+)\)$" % locals(), re.DOTALL | re.IGNORECASE | re.UNICODE);
	counts = [];
	for fileName in fileList :
		m = extractDupCountRe.search(fileName);
		if m :
			buffer = m.group(u'number')
			buffer = int(buffer)
			counts.append(buffer);
	
	#중복 숫자를 설정
	if counts :
		counts.sort();
		notDuplicatedNumber = counts[-1] + 1;
	else :
		notDuplicatedNumber = 1;
		
	#중복 회피 이름 생성
	if spliteExt :
		uniqueName = u"%(targetFileName)s (d%(notDuplicatedNumber)d).%(targetFileExt)s" % locals();
	else :
		uniqueName = u"%(targetFileName)s (d%(notDuplicatedNumber)d)" % locals();
	
	return os.path.join(targetDirname, uniqueName);

def filename(filePath) :
	"""
	@brief 파일 경로로 부터 확장자를 제외한 파일명을 추출해냅니다.
	@param filePath 주어지는 값은 유니코드 문자열이여야 함.\n
		예를 들어, `../asd/.qwe.tar.bz2'가 인자로 주어진 다면 반환값은 `.qwe.tar' 입니다.
	"""
	rx = re.compile(ur"^(.*/)?(?P<name_space>.+?)?(?P<ext_space>\.[a-z0-9]+)?$", re.DOTALL | re.IGNORECASE | re.UNICODE);
	
	result = rx.search(filePath)
	if not result:
		return unicode()
		
	nameSpace = result.group('name_space');
	if nameSpace :
		return nameSpace;
	
	extSpace = result.group('ext_space')
	if extSpace:
		return extSpace;
		
	return unicode()

def extension(fileName) :
	"""
	@brief 주어진 파일명의 확장자를 추출합니다.
	@param fileName 주어지는 값은 유니코드 문자열이여야 함.
	@return `../asd/.qwe'가 인자로 주어진 다면 반환값은 (빈 값) 입니다.
	@return 만약 확장자가 없다면, (빈 값)을 리턴합니다.
	"""
	extRe = re.compile(ur"[^/]+\.(?P<ext>[a-z0-9]+)$", re.DOTALL | re.IGNORECASE | re.UNICODE);
	result = extRe.search(fileName);
	if result :
		return result.group('ext');
	return unicode();



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import PIL.Image
import PIL.ImageChops
import math
import copy

RGB_MIN_VALUE = 0
RGB_MAX_VALUE = 256

#PIL.Image의 L 모드를 의미.
GRAYSCALE_MIN_VALUE = 0
GRAYSCALE_MAX_VALUE = 256

def quantizeByImprovedGrayScale(image, colorCount=GRAYSCALE_MAX_VALUE):
	"""
	@brief 주어진 이미지에 대해 Improved Gray Scale(IGS) 양자화시킵니다.
	@link http://stackoverflow.com/questions/14434963/change-number-of-gray-levels-in-a-grayscale-image-in-matlab change number of gray levels in a grayscale image in matlab - Stack Overflow
	@link http://code.activestate.com/recipes/259112/ Improved Gray Scale Quantization (Python recipe)
	@throw ValueError 사용 불가능한 모드의 이미지를 인자로 준 경우
	@param image PIL.Image 이미지 객체\n
		이 이미지는 L 모드여야 합니다.
	@param colorCount 원본의 색상 수\n
		주어진 이미지의 색상 수를 목표로 하는 색상로 맞추는데 사용 할 수도 있습니다. 이 경우, 주어진 모드가 한 채널에서 가질 수 있는 최대 값을 상한선으로 하여, 목표로 하는 색상의 수를 정하십시오.
	@return 변경된 PIL.Image 이미지 객체
	@remark 언제나 주어진 이미지 객체를 복제한 객체를 수정하여 반환합니다.
	"""
	if image.mode != 'L':
		raise ValueError('{0} 모드의 이미지는 사용 할 수 없습니다.'.format(image.mode))
	
	result = image.copy()
	raito = GRAYSCALE_MAX_VALUE / colorCount
	mask = math.trunc(raito - 1)
	prev_sum = 0
	width, height = image.size
	for x in xrange(0, width):
		for y in xrange(0, height):
			value = image.getpixel((x, y))
			if math.trunc(value/raito) != mask:
				prev_sum = value + (prev_sum & mask)
			else:
				prev_sum = value
				pass
			buffer = math.trunc(prev_sum/raito) * raito
			result.putpixel((x, y), buffer)
			pass
		pass
	return result

def changeColorDepth(image, colorCount):
	"""
	@brief 이미지의 색 깊이를 감소시킵니다.
	@link http://en.wikipedia.org/wiki/List_of_monochrome_and_RGB_palettes List of monochrome and RGB palettes - Wikipedia, the free encyclopedia
	@link http://stackoverflow.com/questions/14434963/change-number-of-gray-levels-in-a-grayscale-image-in-matlab change number of gray levels in a grayscale image in matlab - Stack Overflow
	@throw ValueError 사용 불가능한 모드의 이미지를 인자로 준 경우
	@param image PIL.Image 객체\n
		L, RGB, RGBA 모드의 이미지를 처리 할 수 있습니다.
	@param colorCount 결과물의 색상 수\n
		주어진 모드가 한 채널에서 가질 수 있는 최대 값을 상한선으로 하여, 목표로 하는 색상의 수를 정하십시오.
	@return 변경된 PIL.Image 이미지 객체
	@remark 언제나 주어진 이미지 객체를 복제한 객체를 수정하여 반환합니다.
	"""
	if image.mode == 'L':
		raito = GRAYSCALE_MAX_VALUE / colorCount
		change = lambda value: math.trunc(value/raito)*raito
	elif image.mode == 'RGB' or image.mode == 'RGBA':
		raito = RGB_MAX_VALUE / colorCount
		change = lambda value: tuple(map(lambda value: math.trunc(value/raito)*raito, value))
	else:
		raise ValueError('{0} 모드의 이미지는 사용 할 수 없습니다.'.format(image.mode))
	
	result = image.copy()
	width, height = image.size
	for x in xrange(0, width):
		for y in xrange(0, height):
			value = image.getpixel((x, y))
			buffer = change(value)
			result.putpixel((x, y), buffer)
			pass
		pass
	return result

def mostPopularEdgeColor(image):
	"""
	@brief 이미지의 둘레(left,right,top,bottom)에서 가장 많이 나타나는 색깔이 무엇인지 계산한다.
	@link http://coreapython.hosting.paran.com/howto/sebsauvage_net-%20Snyppets%20-%20Python%20snippets.htm#autocrop 이 함수에 대해 참조한 자료가 있는 웹페이지
	@param image PIL 이미지 객체
	@return 가장 많은 색상(정수 튜플 (R,G,B))
	"""
	if image.mode != 'RGB':
		image = image.convert("RGB")
		pass
	
	# 이미지의 둘레에서 픽셀을 얻는다:
	width,height = image.size
	left   = image.crop((0, 1, 1, height-1))
	right  = image.crop((width-1, 1, width, height-1))
	top    = image.crop((0, 0, width, 1))
	bottom = image.crop((0, height-1, width, height))
	pixels = left.tostring() + right.tostring() + top.tostring() + bottom.tostring()

	# 어떤 RGB 삼중원소가 가장 많은지 계산한다
	counts = dict()
	for i in range(0, len(pixels), 3):
		rgb = pixels[i] + pixels[i+1] + pixels[i+2]
		if rgb in counts:
			counts[rgb] += 1
		else:
			counts[rgb] = 1
			pass
		pass
	
	# 가장 많은 색깔을 얻는다:        
	mostPopularColor = sorted([(count, rgba) for (rgba, count) in counts.items()], reverse=True)[0][1]
	r = ord(mostPopularColor[0])
	g = ord(mostPopularColor[1])
	b = ord(mostPopularColor[2])
	return (r, g, b)

def trim(image, backgroundColor=None, fuzz=0):
	"""
	@brief 이미지 주위에 있는 여백을 제거한다.
	@details \n
		이미지에 알파 채널이 있으면, 그것을 사용하여 무엇을 자를지 고른다. 그렇지 않으면, 이미지 둘레에서 가장 많은 색상을 찾으려고 시도한다. 그리고 이 색상을 여백으로 간주한다. (backgroundColor 매개변수로 이 색상을 오버라이드할 수 있다.)
	
	@warning fuzz옵션 사용시, 이미지가 여백과 유사한 색상일 경우 이미지의 경계를 넘어 이미지까지 잘리는 문제가 생길 수 있다.
	@warning fuzz옵션 사용시, 여백에 잡음이 존재하는 경우 여백을 넘어 이미지를 자를 수 있습니다.
	
	@throw ValueError fuzz값이 잘못된 경우
	@throw ValueError 주어진 이미지가 처리 불가능한 모드인 경우
	
	@link http://coreapython.hosting.paran.com/howto/sebsauvage_net-%20Snyppets%20-%20Python%20snippets.htm#autocrop 이 함수에 대해 참조한 자료가 있는 웹페이지
	@link http://stackoverflow.com/questions/10615901/trim-whitespace-using-pil 이 함수에 대해 참조한 자료가 있는 웹페이지
	@link https://pillow.readthedocs.org/reference/ImageChops.html Pillow 사용자 문서 - ImageChops 모듈 API
	
	@param image (PIL 이미지 객체) 다듬을 이미지.\n
		알파 채널이 포함된 이미지 또는, RGB 형식의 이미지만 처리 할 수 있습니다.
	@param backgroundColor "다듬을 배경"으로 간주될 색상(RGB). (3개의 정수가 담긴 터플)\n
		예컨데, tuple(0,0,255)와 같은 형식으로 주어야 합니다.\n
		이미지가 투명이면, 이 매개변수는 무시된다.\n
		이미지가 투명이 아니고 이 매개변수가 주어지지 않으면, 자동으로 계산된다.
	@param fuzz float 또는 int형(0~99.9). 두 색상을 동일하게 취급하는 정도를 설정합니다.\n
		예컨데, 0은 각 픽셀이 가진 값 그대로 비교하고, 1은 RGB 1,1,1과 2,2,2를 동일하게 취급합니다. 숫자가 커질 수록 범위가 증가합니다.
	
	@return 다듬어진 이미지(PIL 이미지 객체).
	@return 변경될 내용이 없는 경우, 원본을 반환합니다.
	"""
	bbox = None
	
	#fuzz값 체크
	if not isinstance(fuzz, (float, int)) or not 0 <= fuzz or not fuzz <= 99.9:
		raise ValueError('fuzz에는 오직 0~99사이의 int 또는 float만 사용 할 수 있습니다.')
	
	#이미지에 투명 레이어가 있으면, 그걸 사용한다.
	if 'A' in image.getbands():
		#이는 투명 레이어가 있는 모든 모드에 작동한다
		bbox = image.split()[list(image.getbands()).index('A')].getbbox()
	#RGB 모드인 경우
	elif image.mode == 'RGB':
		if not backgroundColor:
			backgroundColor = mostPopularEdgeColor(image)
			pass
		# 불투명 이미지를 다듬는다. .getbbox()는 언제나 검정색을 다듬는다. 그래서 이미지에서 "배경색"을 빼줄 필요가 있다.
		bg = PIL.Image.new(image.mode, image.size, backgroundColor)
		diff = PIL.ImageChops.difference(image, bg)  # 이미지에서 배경색을 뺀다.
		
		#차이를 감소시킨다.
		if fuzz > 0:
			buffer = math.trunc(RGB_MAX_VALUE * fuzz / 100)
			offset = - buffer
			diff = PIL.ImageChops.add(diff, diff, 2.0, offset)
			pass
		
		bbox = diff.getbbox()  # 이미지의 진짜 둘레를 찾는다.
	else:
		raise ValueError('{0} 모드의 이미지는 처리가 불가능 합니다.'.format(image.mode))
	
	if bbox:
		image = image.crop(bbox)
		pass
	
	return image
